@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,8 +13,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-import proto  # type: ignore
+from typing import MutableMapping, MutableSequence
 
+import proto  # type: ignore
 
 __protobuf__ = proto.module(
     package="google.cloud.apigeeconnect.v1",
@@ -49,16 +50,25 @@ class ListConnectionsRequest(proto.Message):
             page token.
     """
 
-    parent = proto.Field(proto.STRING, number=1,)
-    page_size = proto.Field(proto.INT32, number=2,)
-    page_token = proto.Field(proto.STRING, number=3,)
+    parent: str = proto.Field(
+        proto.STRING,
+        number=1,
+    )
+    page_size: int = proto.Field(
+        proto.INT32,
+        number=2,
+    )
+    page_token: str = proto.Field(
+        proto.STRING,
+        number=3,
+    )
 
 
 class ListConnectionsResponse(proto.Message):
     r"""The response for [ListConnections][Management.ListConnections].
 
     Attributes:
-        connections (Sequence[google.cloud.apigeeconnect_v1.types.Connection]):
+        connections (MutableSequence[google.cloud.apigeeconnect_v1.types.Connection]):
             A list of clients.
         next_page_token (str):
             A token that can be sent as ``page_token`` to retrieve the
@@ -70,8 +80,15 @@ class ListConnectionsResponse(proto.Message):
     def raw_page(self):
         return self
 
-    connections = proto.RepeatedField(proto.MESSAGE, number=1, message="Connection",)
-    next_page_token = proto.Field(proto.STRING, number=2,)
+    connections: MutableSequence["Connection"] = proto.RepeatedField(
+        proto.MESSAGE,
+        number=1,
+        message="Connection",
+    )
+    next_page_token: str = proto.Field(
+        proto.STRING,
+        number=2,
+    )
 
 
 class Connection(proto.Message):
@@ -87,9 +104,19 @@ class Connection(proto.Message):
             The count of streams.
     """
 
-    endpoint = proto.Field(proto.STRING, number=1,)
-    cluster = proto.Field(proto.MESSAGE, number=2, message="Cluster",)
-    stream_count = proto.Field(proto.INT32, number=3,)
+    endpoint: str = proto.Field(
+        proto.STRING,
+        number=1,
+    )
+    cluster: "Cluster" = proto.Field(
+        proto.MESSAGE,
+        number=2,
+        message="Cluster",
+    )
+    stream_count: int = proto.Field(
+        proto.INT32,
+        number=3,
+    )
 
 
 class Cluster(proto.Message):
@@ -102,8 +129,14 @@ class Cluster(proto.Message):
             The region of the cluster.
     """
 
-    name = proto.Field(proto.STRING, number=1,)
-    region = proto.Field(proto.STRING, number=2,)
+    name: str = proto.Field(
+        proto.STRING,
+        number=1,
+    )
+    region: str = proto.Field(
+        proto.STRING,
+        number=2,
+    )
 
 
 __all__ = tuple(sorted(__protobuf__.manifest))
